"""Pylucene utilities.

    >>> init(maxheap='1024m')
    >>> def do_something_with_lucene():
    ...     attach_thread()
    ...     lucene.Document()
    ...     detach_thread()
    >>> thread = threading.Thread(target=do_something_with_lucene)
    >>> thread.start()
    >>> thread.join()
    >>> init(maxheap='256m')
    >>> init(maxheap='1024m')

"""

import lucene
import threading
import logging
import warnings

def _logger():
    return logging.getLogger('rest-ws.util.pylucene')

def attach_thread():
    """attach the current thread to pylucene's JVM"""
    logger = _logger()
    logger.info('attaching pylucene thread')
    t = threading.currentThread()
    lucene.getVMEnv().attachCurrentThread(t.getName(), t.isDaemon())
    logger.info('pylucene thread attached')

def detach_thread():
    """detach the current thread from pylucene's JVM"""
    logger = _logger()
    logger.info('detaching pylucene thread')
    lucene.getVMEnv().detachCurrentThread()
    logger.info('pylucene thread detached')

class _init:
    def __init__(self):
        self._init_lucene_args = None
    
    def __call__(self, **args):
        logger = _logger()
        if not lucene.getVMEnv():
            logger.info('initializing pylucene with args=%s' % args)
            lucene.initVM(lucene.CLASSPATH, **args)
            logger.info('pylucene initialized')
            self._init_lucene_args = args
        else:
            warnings.warn('ignoring additional pylucene init', RuntimeWarning)
            if args != self._init_lucene_args:
                message = 'original lucene args (%s) are different to the new ones (%s), which have been ignored' % (self._init_lucene_args, args)
                warnings.warn(message, RuntimeWarning)

init = _init()
"""initialize pylucene's JVM"""

def __indexsearcher_fscheck_monkey_patch():
    """"Monkey-patch pylucene's IndexSearcher to check for directory existence
    before creation.
    
        >>> import os
        >>> init()
        >>> __indexsearcher_fscheck_monkey_patch()
        >>> wrongpath = '/tmp/idontexist'
        >>> while os.path.exists(wrongpath): wrongpath += '!'
        >>> lucene._IndexSearcher_nopatch(wrongpath) #doctest: +ELLIPSIS
        Traceback (most recent call last):
          ...
        JavaError: java.io.FileNotFoundException: ...@/tmp/idontexist...: files:
        >>> os.path.exists(wrongpath)
        True
        >>> os.rmdir(wrongpath)
        >>> lucene.IndexSearcher(wrongpath) #doctest: +ELLIPSIS
        Traceback (most recent call last):
          ...
        IOError: directory '/tmp/idontexist...' does not exist
        >>> os.path.exists(wrongpath)
        False
    
    """
    logger = _logger()
    if hasattr(lucene, '_IndexSearcher_nopatch'):
        logger.debug('lucene.IndexSearcher already monkey-patched, skipping')
        return
    import os
    logger.debug('monkey-patching lucene.IndexSearcher')
    lucene._IndexSearcher_nopatch = lucene.IndexSearcher
    class FSCheckSearcher(lucene._IndexSearcher_nopatch):
        def __init__(self, dir):
            if isinstance(dir, basestring):
                if not os.path.exists(dir):
                    raise IOError("directory '%s' does not exist" % dir)
            lucene._IndexSearcher_nopatch.__init__(self, dir)
    lucene.IndexSearcher = FSCheckSearcher
    logger.debug('lucene.IndexSearcher monkey-patched')

__indexsearcher_fscheck_monkey_patch()

__all__ = ['attach_thread', 'detach_thread', 'init']
