class PyConfigException(Exception):
    """pyconfig Error"""

class PyConfig(dict):
    def __init__(self, values, filenames):
        dict.__init__(self, values)
        self.filenames = filenames

def read(filename, defaults={}):
    """Read configuration from a given filename, with optional defaults.
    
        >>> from pprint import pprint
        >>> from tempfile import NamedTemporaryFile as temp
        >>> tf = temp()
        >>> tf.file.write(\"\"\"
        ... y = 1
        ... z = dict(z1=1, z2=2)
        ... \"\"\")
        >>> tf.file.flush()
        >>> pprint(read(tf.name, dict(x=0, y=0, z=dict(z0=0, z1=0))), width=1)
        {'x': 0,
         'y': 1,
         'z': {'z0': 0,
               'z1': 1,
               'z2': 2}}
    
    Defaults are available as variables::
        
        >>> tf = temp()
        >>> tf.file.write(\"\"\"
        ... message = 'Hello, %(name)s!'%globals()
        ... x_plus_five = x + 5
        ... \"\"\")
        >>> tf.file.flush()
        >>> pprint(read(tf.name, dict(name='Terry', x=5)), width=1)
        {'message': 'Hello, Terry!',
         'name': 'Terry',
         'x': 5,
         'x_plus_five': 10}
    
    Names starting with '_' are private::
        
        >>> tf = temp()
        >>> tf.file.write(\"\"\"
        ... public = 'OK'
        ... _private = 'secret message'
        ... \"\"\")
        >>> tf.file.flush()
        >>> pprint(read(tf.name))
        {'public': 'OK'}
    
    Problems evaluating config files are wrapped in PyConfigException, except
    for IOErrors::
    
        >>> tf = temp()
        >>> tf.file.write("a=")
        >>> tf.file.flush()
        >>> read(tf.name) #doctest: +ELLIPSIS +NORMALIZE_WHITESPACE
        Traceback (most recent call last):
          ...
        PyConfigException: Error evaluating config (...): invalid syntax
        (..., line 1)
        >>> tf = temp()
        >>> tf.file.write("raise Exception('I am a bad config file')")
        >>> tf.file.flush()
        >>> read(tf.name) #doctest: +ELLIPSIS
        Traceback (most recent call last):
          ...
        PyConfigException: Error evaluating config (...): I am a bad config file
        >>> read('/tmp/idontexist!!!')
        Traceback (most recent call last):
          ...
        IOError: [Errno 2] No such file or directory: '/tmp/idontexist!!!'
    
    """
    config = dict(defaults)
    globals = dict(config)
    try:
        execfile(filename, globals)
    except IOError:
        raise
    except Exception, e:
        raise PyConfigException('Error evaluating config (%s): %s' %
                                (filename, str(e)))
    for key in globals.keys():
        if key.startswith('_'):
            globals.pop(key)
    _deep_update(config, globals)
    return PyConfig(config, [filename])

def readmulti(filenames, defaults={}):
    """Read configuration from a list of possible candidates, with optional
    defaults.
    
    Returns the final config object and a list of the successfully read files.
    
        >>> from pprint import pprint
        >>> from tempfile import NamedTemporaryFile as temp
        >>> tf1 = temp()
        >>> tf1.file.write(\"\"\"
        ... b = 1
        ... c = 1
        ... \"\"\")
        >>> tf1.file.flush()
        >>> tf2 = temp()
        >>> tf2.file.write(\"\"\"
        ... c = 2
        ... d = 3
        ... \"\"\")
        >>> tf2.file.flush()
        >>> config, filenames = readmulti([tf1.name, tf2.name,
        ...                                'idontexist!!!=!=!=!'],
        ...                               dict(a=0, b=0))
        >>> pprint(config, width=1)
        {'a': 0,
         'b': 1,
         'c': 2,
         'd': 3}
        >>> filenames == [tf1.name, tf2.name]
        True
    
    """
    readfilenames = list()
    config = dict(defaults)
    for filename in filenames:
        try:
            config = read(filename, config)
            readfilenames.append(filename)
        except IOError:
            pass
    return PyConfig(config, readfilenames), readfilenames

def _deep_update(dst, src):
    """deep update of (dict) dst from src"""
    for key, val in src.items():
        if isinstance(val, dict) and key in dst and isinstance(dst[key], dict):
            _deep_update(dst[key], val)
        else:
            dst[key] = val
