import re

class FilenameScheme:
    """Tag guessing scheme.
    
        >>> sch = FilenameScheme("%artist/%album/[%track] %title")
        >>> sch.guess_tags("~/Music/Tool/Lateralus/[1] The Grudge.mp3")
        {'track': 'The Grudge', 'release': 'Lateralus', 'track_number': '1', 'artist': 'Tool'}
    """
    def __init__(self, s):
        self.artist_field  = -1
        self.title_field   = -1
        self.track_field   = -1
        self.album_field   = -1
        self.comment_field = -1
        
        artist  = s.find("%artist")
        title   = s.find("%title")
        track   = s.find("%track")
        album   = s.find("%album")
        comment = s.find("%comment")
        
        field_number = 0
        i = s.find("%")
        while i!=-1:
            if artist==i:
                self.artist_field = field_number
            if title==i:
                self.title_field = field_number
            if track==i:
                self.track_field = field_number
            if album==i:
                self.album_field = field_number
            if comment==i:
                self.comment_field = field_number
            
            field_number += 1
            i = s.find('%', i + 1)
        
        self.regexp = re.compile(self.compose_regexp(s))
    
    def compose_regexp(self, s):
        s = s.strip()
        regexp = re.escape(s)
        regexp = r".*?" + regexp
        regexp = regexp.replace(r'\ ', r'\s+')
        regexp = regexp.replace(r'\/', r'[\/\\]')
        regexp = regexp.replace(r'\%title',  r"([\w\s\'\&\_\,\.]+)")
        regexp = regexp.replace(r'\%artist', r"([\w\s\'\&\_\,\.]+)")
        regexp = regexp.replace(r'\%album',  r"([\w\s\'\&\_\,\.]+)")
        regexp = regexp.replace(r'\%track',  r'(\d+)')
        regexp = regexp.replace(r'\%comment',r'([\w\s\_]+)')
        regexp += r'[^/]*$'
        return regexp
    
    def guess_tags(self, filename):
        filename = filename.strip()
        try:
            filename = filename[:filename.rindex('.')]
        except ValueError:
            pass
        m = self.regexp.match(filename)
        if m is None:
            return None
        g = m.groups()
        tags = dict()
        if self.artist_field!=-1:
            tags['artist'] = g[self.artist_field]
        if self.title_field!=-1:
            tags['track'] = g[self.title_field]
        if self.track_field!=-1:
            tags['track_number'] = g[self.track_field]
        if self.album_field!=-1:
            tags['release'] = g[self.album_field]
        if self.comment_field!=-1:
            tags['comment'] = g[self.comment_field]
        return tags

_SCHEMES = [
    FilenameScheme(s) for s in [
        "%track - %artist - %title [%comment]",
        "%track - %artist - %title (%comment)",
        "%track - %artist - %title",
        "%track. %artist - %title [%comment]",
        "%track. %artist - %title (%comment)",
        "%track. %artist - %title",
        #"%track - %title",
        "%artist - (%track) - %title [%comment]",
        "%artist - (%track) - %title (%comment)",
        "%artist - (%track) - %title",
        "%artist - [%track] - %title [%comment]",
        "%artist - [%track] - %title (%comment)",
        "%artist - [%track] - %title",
        "%artist - %track - %title [%comment]",
        "%artist - %track - %title (%comment)",
        "%artist - %track - %title",
        "[%track] %artist - %title [%comment]",
        "[%track] %artist - %title (%comment)",
        "[%track] %artist - %title",
        "%track %artist - %title [%comment]",
        "%track %artist - %title (%comment)",
        "%track %artist - %title",
        "(%artist) %title [%comment]",
        "(%artist) %title (%comment)",
        "(%artist) %title",
        "%artist - %title [%comment]",
        "%artist - %title (%comment)",
        "%artist - %title",
        "%artist %title [%comment]",
        "%artist %title (%comment)",
        "%artist %title",
        "%artist/%album/[%track] %title [%comment]",
        "%artist/%album/[%track] %title (%comment)",
        "%artist/%album/[%track] %title",
        "%artist/%album/%track - %title [%comment]",
        "%artist/%album/%track - %title (%comment)",
        "%artist/%album/%track - %title",
        "%artist/%album/%track. %title [%comment]",
        "%artist/%album/%track. %title (%comment)",
        "%artist/%album/%track. %title",
        "%artist - %album/[%track] %title [%comment]",
        "%artist - %album/[%track] %title (%comment)",
        "%artist - %album/[%track] %title",
        "%artist - %album/%track - %title [%comment]",
        "%artist - %album/%track - %title (%comment)",
        "%artist - %album/%track - %title",
        "%artist - %album/%track. %title [%comment]",
        "%artist - %album/%track. %title (%comment)",
        "%artist - %album/%track. %title",
    ]
]

def guess_tags(filename, schemes=_SCHEMES):
    """Guess tags for a given filename, using the default set of schemes.
    
        >>> guess_tags('~/Music/[01] Pearl Jam - Sometimes.mp3')
        {'track': 'Sometimes', 'track_number': '01', 'artist': 'Pearl Jam'}
        >>> guess_tags(r'/Music/Pearl Jam/No Code/[01] Sometimes.mp3')
        {'track': 'Sometimes', 'release': 'No Code', 'track_number': '01', 'artist': 'Pearl Jam'}
        >>> guess_tags("~/Music/Tool/Lateralus/1. The Grudge.mp3")
        {'track': 'The Grudge', 'release': 'Lateralus', 'track_number': '1', 'artist': 'Tool'}
        >>> guess_tags("~/Music/Tool/Lateralus/1 - The Grudge.mp3")
        {'track': 'The Grudge', 'release': 'Lateralus', 'track_number': '1', 'artist': 'Tool'}
        >>> guess_tags("~/Music/Tool - Lateralus/1. The Grudge.mp3")
        {'track': 'The Grudge', 'release': 'Lateralus', 'track_number': '1', 'artist': 'Tool'}
    
    """
    best_tags = None
    for scheme in _SCHEMES:
        tags = scheme.guess_tags(filename)
        if tags is not None:
            if best_tags is None or len(tags) > len(best_tags):
                best_tags = tags
    return best_tags
